<?php
/********************************************************************
Product		: Payage
Date		: 2 June 2017
Copyright	: Les Arbres Design 2014-2017
Contact		: http://www.lesarbresdesign.info
Licence		: GNU General Public License
*********************************************************************/
defined('_JEXEC') or die('Restricted Access');

	require_once JPATH_ADMINISTRATOR.'/components/com_payage/helpers/payage_helper.php';
	require_once JPATH_ADMINISTRATOR.'/components/com_payage/helpers/db_helper.php';
	require_once JPATH_ADMINISTRATOR.'/components/com_payage/helpers/trace_helper.php';
	require_once JPATH_ADMINISTRATOR.'/components/com_payage/models/account.php';
	require_once JPATH_ADMINISTRATOR.'/components/com_payage/models/payment.php';
	$app = JFactory::getApplication();

	LAPG_trace::trace('Gateway '.$_SERVER['REQUEST_METHOD'].' request: Get data: '.print_r($_GET,true).' Post data: '.print_r($_POST,true));

// load the Payage language file (we only install it in the back end)

	$lang = JFactory::getLanguage();
	$lang->load('com_payage', JPATH_ADMINISTRATOR.'/components/com_payage');

// this entry point should only be called by a payment gateway and should always include an account id and a transaction id
// However: Just sometimes, PayPal only returns the first URL parameter.
// In order to recover from this, the PayPal addon version 1.10 places the AID and the TID in the 'custom' parameter ...

	$jinput = JFactory::getApplication()->input;
	$aid = $jinput->get('aid','', 'STRING');
	$tid = $jinput->get('tid','', 'STRING');
	if (($aid == '') or ($tid == ''))	    // if either is missing this is not a valid entry
        {                                   // however, we have seen some cases where PayPal returns without the expected parameters ...
		LAPG_trace::trace("Front end entry without AID or TID");
        $custom = $jinput->get('custom','', 'STRING');          // do we have a 'custom' parameter? (IPN)
        $jinput->set('task','notify');                          // if 'custom' is set, this is an IPN notify
		LAPG_trace::trace(" - custom = $custom");
        if ($custom == '')
            {
            $custom = $jinput->get('cm','', 'STRING');          // do we have a 'cm' parameter? (website return)
            $jinput->set('task','return');                      // if 'cm' is set, this is a website return
    		LAPG_trace::trace(" - cm = $custom");
            }
        $ppta = substr($custom,0,4);            // first 4 characters should be 'PPTA'
        $tid = substr($custom,5,32);            // TID is next 32 characters after the comma
        $aid = substr($custom,38);              // AID is remaining chars
        if ( ($ppta != 'PPTA') or (strlen($tid) != 32) or (!is_numeric($aid)) )
            {    
    		LAPG_trace::trace(" - unable to recover");
            $params = JComponentHelper::getParams('com_payage');		// get component parameters
            $error_email = $params->get('error_email','');
            if ($error_email != '')
                {
                $subject = "Invalid call to Payage on ".$app->get('sitename');
                $body = ' Payage was called with invalid entry parameters at '.date("Y/m/d H:i").'<br />';	
                $body .= "The call could not be processed by Payage because there was no account id and/or no transaction id<br />";
                $body .= "Server details: <pre>".print_r($_SERVER,true).'</pre><br />';
                $body .= "Get data: <pre>".print_r($_GET,true).'</pre><br />';
                if (!empty($_POST))
                    $body .= "Post data: <pre>".print_r($_POST,true).'</pre><br />';
                PayageHelper::send_email($error_email, $subject, $body);
                $app->redirect(JURI::root());       // redirect to home page
                return;
                }
            }
		LAPG_trace::trace(" - recovered: AID = $aid and TID = $tid");
        }

	$account_model = new PayageModelAccount;
	$account_data = $account_model->getOne($aid);
	if ($account_data === false)
		{
		LAPG_trace::trace("Account $aid not found");
		$app->redirect(JURI::root());
		}
	$payment_model = new PayageModelPayment;
	$payment_data = $payment_model->getOne($tid,'pg_transaction_id');
	if (($payment_data === false) and (!isset($account_model->specific_data->tid_optional)))	// see the WorldPay Online gateway model
		{
		LAPG_trace::trace("Payment $tid not found");
		$app->redirect(JURI::root());
		}
	
// we have a valid request with a correct account id and transaction id
// get the gateway info for the relevant account and create an instance of the relevant model

	$gateway_model = PayageHelper::getGatewayInstance($account_data->gateway_type);

// call the gateway model to handle the request

	$gateway_model->getOne($aid);
	
	$ret = $gateway_model->Gateway_handle_request($payment_model);
	switch ($ret)
		{
		case LAPG_CALLBACK_NONE:		// end the gateway request with no external actions 
			return;
			
		case LAPG_CALLBACK_CANCEL;		// redirect the request to the Payage application that originated the transaction
		case LAPG_CALLBACK_USER:		// - this is used to return to the application after an online payment transaction 
			if (strstr($payment_data->app_return_url,'?'))
				$url = $payment_data->app_return_url.'&tid='.$payment_data->pg_transaction_id;
			else
				$url = $payment_data->app_return_url.'?tid='.$payment_data->pg_transaction_id;
			LAPG_trace::trace('Redirecting to '.$url);
			$app->redirect($url);
			return;
			
		case LAPG_CALLBACK_UPDATE:		// call an external PHP file, for example after a notification from a gateway
			if (empty($payment_data->app_update_path))
				{
				LAPG_trace::trace("Update callback but no app_update_path for tid ".$payment_data->pg_transaction_id);
				return;
				}

			if (!file_exists($payment_data->app_update_path))
				{
				LAPG_trace::trace("Update callback cannot find file ".$payment_data->app_update_path.' - tid '.$payment_data->pg_transaction_id);
				return;
				}
			require_once $payment_data->app_update_path;
			LAPG_trace::trace("Calling payment_update() for tid ".$payment_data->pg_transaction_id.' '.$payment_data->app_update_path);
			payment_update($payment_data->pg_transaction_id);
			return;
		}

